@@ -1,19 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2000-2001,2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "libxfs.h"
@@ -41,12 +29,9 @@ const field_t	dir2sf_flds[] = {
 	{ NULL }
 };
 
-#define UOFF(f)	bitize(offsetof(xfs_dir2_inou_t, f))
 const field_t	dir2_inou_flds[] = {
-	{ "i8", FLDT_DIR2_INO8, OI(UOFF(i8)), dir2_inou_i8_count, FLD_COUNT,
-	  TYP_INODE },
-	{ "i4", FLDT_DIR2_INO4, OI(UOFF(i4)), dir2_inou_i4_count, FLD_COUNT,
-	  TYP_INODE },
+	{ "i8", FLDT_DIR2_INO8, NULL, dir2_inou_i8_count, FLD_COUNT, TYP_INODE},
+	{ "i4", FLDT_DIR2_INO4, NULL, dir2_inou_i4_count, FLD_COUNT, TYP_INODE},
 	{ NULL }
 };
 
@@ -110,9 +95,7 @@ dir2_inou_size(
 	ASSERT(bitoffs(startoff) == 0);
 	ASSERT(idx == 0);
 	sf = (struct xfs_dir2_sf_hdr *)XFS_DFORK_DPTR(dip);
-	return bitize(sf->i8count ?
-		      (uint)sizeof(xfs_dir2_ino8_t) :
-		      (uint)sizeof(xfs_dir2_ino4_t));
+	return bitize(sf->i8count ? XFS_INO64_SIZE : XFS_INO32_SIZE);
 }
 
 static int
