@@ -246,6 +246,7 @@ zero_btree_node(
 	xfs_alloc_key_t		*akp;
 	char			*zp1, *zp2;
 	char			*key_end;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 	nrecs = be16_to_cpu(block->bb_numrecs);
 	if (nrecs < 0)
@@ -265,11 +266,11 @@ zero_btree_node(
 		break;
 	case TYP_INOBT:
 	case TYP_FINOBT:
-		if (nrecs > mp->m_inobt_mxr[1])
+		if (nrecs > igeo->inobt_mxr[1])
 			return;
 
 		ikp = XFS_INOBT_KEY_ADDR(mp, block, 1);
-		ipp = XFS_INOBT_PTR_ADDR(mp, block, 1, mp->m_inobt_mxr[1]);
+		ipp = XFS_INOBT_PTR_ADDR(mp, block, 1, igeo->inobt_mxr[1]);
 		zp1 = (char *)&ikp[nrecs];
 		zp2 = (char *)&ipp[nrecs];
 		key_end = (char *)ipp;
@@ -328,7 +329,7 @@ zero_btree_leaf(
 		break;
 	case TYP_INOBT:
 	case TYP_FINOBT:
-		if (nrecs > mp->m_inobt_mxr[0])
+		if (nrecs > M_IGEO(mp)->inobt_mxr[0])
 			return;
 
 		irp = XFS_INOBT_REC_ADDR(mp, block, 1);
@@ -1638,11 +1639,34 @@ process_dir_data_block(
 	}
 }
 
-static void
+static int
 process_symlink_block(
-	char			*block)
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
+	typnm_t		btype,
+	xfs_fileoff_t	last)
 {
-	char *link = block;
+	struct bbmap	map;
+	char		*link;
+	int		ret = 0;
+
+	push_cur();
+	map.nmaps = 1;
+	map.b[0].bm_bn = XFS_FSB_TO_DADDR(mp, s);
+	map.b[0].bm_len = XFS_FSB_TO_BB(mp, c);
+	set_cur(&typtab[btype], 0, 0, DB_RING_IGN, &map);
+	if (!iocur_top->data) {
+		xfs_agnumber_t	agno = XFS_FSB_TO_AGNO(mp, s);
+		xfs_agblock_t	agbno = XFS_FSB_TO_AGBNO(mp, s);
+
+		print_warning("cannot read %s block %u/%u (%llu)",
+				typtab[btype].name, agno, agbno, s);
+		if (stop_on_read_error)
+			ret = -1;
+		goto out_pop;
+	}
+	link = iocur_top->data;
 
 	if (xfs_sb_version_hascrc(&(mp)->m_sb))
 		link += sizeof(struct xfs_dsymlink_hdr);
@@ -1660,6 +1684,12 @@ process_symlink_block(
 		if (zlen < mp->m_sb.sb_blocksize)
 			memset(link + linklen, 0, zlen);
 	}
+
+	iocur_top->need_crc = 1;
+	ret = write_buf(iocur_top);
+out_pop:
+	pop_cur();
+	return ret;
 }
 
 #define MAX_REMOTE_VALS		4095
@@ -1879,10 +1909,6 @@ process_single_fsb_objects(
 			}
 			iocur_top->need_crc = 1;
 			break;
-		case TYP_SYMLINK:
-			process_symlink_block(dp);
-			iocur_top->need_crc = 1;
-			break;
 		case TYP_ATTR:
 			process_attr_block(dp, o);
 			iocur_top->need_crc = 1;
@@ -1911,7 +1937,7 @@ static struct bbmap mfsb_map;
 static int mfsb_length;
 
 static int
-process_multi_fsb_objects(
+process_multi_fsb_dir(
 	xfs_fileoff_t	o,
 	xfs_fsblock_t	s,
 	xfs_filblks_t	c,
@@ -1921,14 +1947,6 @@ process_multi_fsb_objects(
 	char		*dp;
 	int		ret = 0;
 
-	switch (btype) {
-	case TYP_DIR2:
-		break;
-	default:
-		print_warning("bad type for multi-fsb object %d", btype);
-		return -EINVAL;
-	}
-
 	while (c > 0) {
 		unsigned int	bm_len;
 
@@ -1987,6 +2005,37 @@ out_pop:
 	return ret;
 }
 
+static bool
+is_multi_fsb_object(
+	struct xfs_mount	*mp,
+	typnm_t			btype)
+{
+	if (btype == TYP_DIR2 && mp->m_dir_geo->fsbcount > 1)
+		return true;
+	if (btype == TYP_SYMLINK)
+		return true;
+	return false;
+}
+
+static int
+process_multi_fsb_objects(
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
+	typnm_t		btype,
+	xfs_fileoff_t	last)
+{
+	switch (btype) {
+	case TYP_DIR2:
+		return process_multi_fsb_dir(o, s, c, btype, last);
+	case TYP_SYMLINK:
+		return process_symlink_block(o, s, c, btype, last);
+	default:
+		print_warning("bad type for multi-fsb object %d", btype);
+		return -EINVAL;
+	}
+}
+
 /* inode copy routines */
 static int
 process_bmbt_reclist(
@@ -2002,6 +2051,7 @@ process_bmbt_reclist(
 	xfs_fileoff_t		last;
 	xfs_agnumber_t		agno;
 	xfs_agblock_t		agbno;
+	bool			is_multi_fsb = is_multi_fsb_object(mp, btype);
 	int			error;
 
 	if (btype == TYP_DATA)
@@ -2065,11 +2115,12 @@ process_bmbt_reclist(
 		}
 
 		/* multi-extent blocks require special handling */
-		if (btype != TYP_DIR2 || mp->m_dir_geo->fsbcount == 1) {
-			error = process_single_fsb_objects(o, s, c, btype, last);
-		} else {
-			error = process_multi_fsb_objects(o, s, c, btype, last);
-		}
+		if (is_multi_fsb)
+			error = process_multi_fsb_objects(o, s, c, btype,
+					last);
+		else
+			error = process_single_fsb_objects(o, s, c, btype,
+					last);
 		if (error)
 			return 0;
 	}
@@ -2405,10 +2456,11 @@ copy_inode_chunk(
 	int			blks_per_buf;
 	int			inodes_per_buf;
 	int			ioff;
+	struct xfs_ino_geometry *igeo = M_IGEO(mp);
 
 	agino = be32_to_cpu(rp->ir_startino);
 	agbno = XFS_AGINO_TO_AGBNO(mp, agino);
-	end_agbno = agbno + mp->m_ialloc_blks;
+	end_agbno = agbno + igeo->ialloc_blks;
 	off = XFS_INO_TO_OFFSET(mp, agino);
 
 	/*
@@ -2420,10 +2472,10 @@ copy_inode_chunk(
 	 * we've been passed (large block sizes can hold multiple inode chunks).
 	 */
 	if (xfs_sb_version_hassparseinodes(&mp->m_sb))
-		blks_per_buf = xfs_icluster_size_fsb(mp);
+		blks_per_buf = igeo->blocks_per_cluster;
 	else
-		blks_per_buf = mp->m_ialloc_blks;
-	inodes_per_buf = min(blks_per_buf << mp->m_sb.sb_inopblog,
+		blks_per_buf = igeo->ialloc_blks;
+	inodes_per_buf = min(XFS_FSB_TO_INO(mp, blks_per_buf),
 			     XFS_INODES_PER_CHUNK);
 
 	/*
@@ -2522,16 +2574,17 @@ scanfunc_ino(
 	int			i;
 	int			numrecs;
 	int			finobt = *(int *) arg;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 	numrecs = be16_to_cpu(block->bb_numrecs);
 
 	if (level == 0) {
-		if (numrecs > mp->m_inobt_mxr[0]) {
+		if (numrecs > igeo->inobt_mxr[0]) {
 			if (show_warnings)
 				print_warning("invalid numrecs %d in %s "
 					"block %u/%u", numrecs,
 					typtab[btype].name, agno, agbno);
-			numrecs = mp->m_inobt_mxr[0];
+			numrecs = igeo->inobt_mxr[0];
 		}
 
 		/*
@@ -2549,14 +2602,14 @@ scanfunc_ino(
 		return 1;
 	}
 
-	if (numrecs > mp->m_inobt_mxr[1]) {
+	if (numrecs > igeo->inobt_mxr[1]) {
 		if (show_warnings)
 			print_warning("invalid numrecs %d in %s block %u/%u",
 				numrecs, typtab[btype].name, agno, agbno);
-		numrecs = mp->m_inobt_mxr[1];
+		numrecs = igeo->inobt_mxr[1];
 	}
 
-	pp = XFS_INOBT_PTR_ADDR(mp, block, 1, mp->m_inobt_mxr[1]);
+	pp = XFS_INOBT_PTR_ADDR(mp, block, 1, igeo->inobt_mxr[1]);
 	for (i = 0; i < numrecs; i++) {
 		if (!valid_bno(agno, be32_to_cpu(pp[i]))) {
 			if (show_warnings)
@@ -2607,7 +2660,7 @@ copy_inodes(
 		levels = be32_to_cpu(agi->agi_free_level);
 
 		finobt = 1;
-		if (!scan_btree(agno, root, levels, TYP_INOBT, &finobt,
+		if (!scan_btree(agno, root, levels, TYP_FINOBT, &finobt,
 				scanfunc_ino))
 			return 0;
 	}
