@@ -99,13 +99,14 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	int		j;
 	int		state;
 	xfs_extlen_t	blen;
+	struct xfs_ino_geometry *igeo = M_IGEO(mp);
 
 	agno = XFS_INO_TO_AGNO(mp, ino);
 	agino = XFS_INO_TO_AGINO(mp, ino);
 	agbno = XFS_INO_TO_AGBNO(mp, ino);
 	*start_ino = NULLFSINO;
 
-	ASSERT(mp->m_ialloc_blks > 0);
+	ASSERT(igeo->ialloc_blks > 0);
 
 	if (agno == mp->m_sb.sb_agcount - 1)
 		max_agbno = mp->m_sb.sb_dblocks -
@@ -123,7 +124,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 * check for the easy case, inodes per block >= XFS_INODES_PER_CHUNK
 	 * (multiple chunks per block)
 	 */
-	if (mp->m_ialloc_blks == 1)  {
+	if (igeo->ialloc_blks == 1)  {
 		if (agbno > max_agbno)
 			return 0;
 		if (check_aginode_block(mp, agno, agino) == 0)
@@ -166,7 +167,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 
 		pthread_mutex_unlock(&ag_locks[agno].lock);
 
-		start_agino = XFS_OFFBNO_TO_AGINO(mp, agbno, 0);
+		start_agino = XFS_AGB_TO_AGINO(mp, agbno);
 		*start_ino = XFS_AGINO_TO_INO(mp, agno, start_agino);
 
 		/*
@@ -196,7 +197,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 		 */
 		start_agbno = rounddown(XFS_INO_TO_AGBNO(mp, ino),
 					fs_ino_alignment);
-		end_agbno = start_agbno + mp->m_ialloc_blks;
+		end_agbno = start_agbno + igeo->ialloc_blks;
 
 		/*
 		 * if this fs has aligned inodes but the end of the
@@ -227,14 +228,14 @@ verify_inode_chunk(xfs_mount_t		*mp,
 		 * ok, put the record into the tree, if no conflict.
 		 */
 		if (find_uncertain_inode_rec(agno,
-				XFS_OFFBNO_TO_AGINO(mp, start_agbno, 0)))
+				XFS_AGB_TO_AGINO(mp, start_agbno)))
 			return(0);
 
-		start_agino = XFS_OFFBNO_TO_AGINO(mp, start_agbno, 0);
+		start_agino = XFS_AGB_TO_AGINO(mp, start_agbno);
 		*start_ino = XFS_AGINO_TO_INO(mp, agno, start_agino);
 
 		irec_p = set_inode_free_alloc(mp, agno,
-				XFS_OFFBNO_TO_AGINO(mp, start_agbno, 0));
+				XFS_AGB_TO_AGINO(mp, start_agbno));
 
 		for (i = 1; i < XFS_INODES_PER_CHUNK; i++)
 			set_inode_free(irec_p, i);
@@ -254,14 +255,14 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 * a discovered inode chunk completely within that range
 	 * would include the inode passed into us.
 	 */
-	if (mp->m_ialloc_blks > 1)  {
-		if (agino > mp->m_ialloc_inos)
-			start_agbno = agbno - mp->m_ialloc_blks + 1;
+	if (igeo->ialloc_blks > 1)  {
+		if (agino > igeo->ialloc_inos)
+			start_agbno = agbno - igeo->ialloc_blks + 1;
 		else
 			start_agbno = 1;
 	}
 
-	end_agbno = agbno + mp->m_ialloc_blks;
+	end_agbno = agbno + igeo->ialloc_blks;
 
 	if (end_agbno > max_agbno)
 		end_agbno = max_agbno;
@@ -271,7 +272,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 */
 	irec_before_p = irec_after_p = NULL;
 
-	find_inode_rec_range(mp, agno, XFS_OFFBNO_TO_AGINO(mp, start_agbno, 0),
+	find_inode_rec_range(mp, agno, XFS_AGB_TO_AGINO(mp, start_agbno),
 		XFS_OFFBNO_TO_AGINO(mp, end_agbno, mp->m_sb.sb_inopblock - 1),
 		&irec_before_p, &irec_after_p);
 
@@ -316,7 +317,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 
 			start_agbno = XFS_AGINO_TO_AGBNO(mp,
 						irec_p->ino_startnum) +
-						mp->m_ialloc_blks;
+						igeo->ialloc_blks;
 
 			/*
 			 * we know that the inode we're trying to verify isn't
@@ -324,7 +325,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 			 * of the gap -- is it within the search range?
 			 */
 			if (irec_next_p != NULL &&
-					agino + mp->m_ialloc_inos >=
+					agino + igeo->ialloc_inos >=
 						irec_next_p->ino_startnum)
 				end_agbno = XFS_AGINO_TO_AGBNO(mp,
 						irec_next_p->ino_startnum);
@@ -339,7 +340,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 * the inode in question and that the space between them
 	 * is too small for a legal inode chunk
 	 */
-	if (end_agbno - start_agbno < mp->m_ialloc_blks)
+	if (end_agbno - start_agbno < igeo->ialloc_blks)
 		return(0);
 
 	/*
@@ -383,7 +384,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 
 	num_blks = chunk_stop_agbno - chunk_start_agbno;
 
-	if (num_blks < mp->m_ialloc_blks || ino_cnt == 0)
+	if (num_blks < igeo->ialloc_blks || ino_cnt == 0)
 		return 0;
 
 	/*
@@ -399,8 +400,8 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 * the chunk
 	 */
 
-	if (num_blks % mp->m_ialloc_blks != 0)  {
-		num_blks = rounddown(num_blks, mp->m_ialloc_blks);
+	if (num_blks % igeo->ialloc_blks != 0)  {
+		num_blks = rounddown(num_blks, igeo->ialloc_blks);
 		chunk_stop_agbno = chunk_start_agbno + num_blks;
 	}
 
@@ -446,7 +447,7 @@ verify_inode_chunk(xfs_mount_t		*mp,
 	 * ok because we'll override the free setting later if the
 	 * contents of the inode indicate it's in use.
 	 */
-	start_agino = XFS_OFFBNO_TO_AGINO(mp, chunk_start_agbno, 0);
+	start_agino = XFS_AGB_TO_AGINO(mp, chunk_start_agbno);
 	*start_ino = XFS_AGINO_TO_INO(mp, agno, start_agino);
 
 	ASSERT(find_inode_rec(mp, agno, start_agino) == NULL);
@@ -607,21 +608,18 @@ process_inode_chunk(
 	xfs_ino_t		ino;
 	int			dirty = 0;
 	int			isa_dir = 0;
-	int			blks_per_cluster;
 	int			cluster_count;
 	int			bp_index;
 	int			cluster_offset;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 	ASSERT(first_irec != NULL);
 	ASSERT(XFS_AGINO_TO_OFFSET(mp, first_irec->ino_startnum) == 0);
 
 	*bogus = 0;
-	ASSERT(mp->m_ialloc_blks > 0);
+	ASSERT(igeo->ialloc_blks > 0);
 
-	blks_per_cluster = mp->m_inode_cluster_size >> mp->m_sb.sb_blocklog;
-	if (blks_per_cluster == 0)
-		blks_per_cluster = 1;
-	cluster_count = XFS_INODES_PER_CHUNK / inodes_per_cluster;
+	cluster_count = XFS_INODES_PER_CHUNK / M_IGEO(mp)->inodes_per_cluster;
 	if (cluster_count == 0)
 		cluster_count = 1;
 
@@ -660,13 +658,16 @@ process_inode_chunk(
 
 		bplist[bp_index] = libxfs_readbuf(mp->m_dev,
 					XFS_AGB_TO_DADDR(mp, agno, agbno),
-					XFS_FSB_TO_BB(mp, blks_per_cluster), 0,
+					XFS_FSB_TO_BB(mp,
+						M_IGEO(mp)->blocks_per_cluster),
+					0,
 					&xfs_inode_buf_ops);
 		if (!bplist[bp_index]) {
 			do_warn(_("cannot read inode %" PRIu64 ", disk block %" PRId64 ", cnt %d\n"),
 				XFS_AGINO_TO_INO(mp, agno, first_irec->ino_startnum),
 				XFS_AGB_TO_DADDR(mp, agno, agbno),
-				XFS_FSB_TO_BB(mp, blks_per_cluster));
+				XFS_FSB_TO_BB(mp,
+					M_IGEO(mp)->blocks_per_cluster));
 			while (bp_index > 0) {
 				bp_index--;
 				libxfs_putbuf(bplist[bp_index]);
@@ -682,8 +683,9 @@ process_inode_chunk(
 		bplist[bp_index]->b_ops = &xfs_inode_buf_ops;
 
 next_readbuf:
-		irec_offset += mp->m_sb.sb_inopblock * blks_per_cluster;
-		agbno += blks_per_cluster;
+		irec_offset += mp->m_sb.sb_inopblock *
+				M_IGEO(mp)->blocks_per_cluster;
+		agbno += M_IGEO(mp)->blocks_per_cluster;
 	}
 	agbno = XFS_AGINO_TO_AGBNO(mp, first_irec->ino_startnum);
 
@@ -728,7 +730,7 @@ next_readbuf:
 			icnt++;
 			cluster_offset++;
 
-			if (icnt == mp->m_ialloc_inos &&
+			if (icnt == igeo->ialloc_inos &&
 					irec_offset == XFS_INODES_PER_CHUNK)  {
 				/*
 				 * done! - finished up irec and block
@@ -743,7 +745,7 @@ next_readbuf:
 				ASSERT(ino_rec->ino_startnum == agino + 1);
 				irec_offset = 0;
 			}
-			if (cluster_offset == inodes_per_cluster) {
+			if (cluster_offset == M_IGEO(mp)->inodes_per_cluster) {
 				bp_index++;
 				cluster_offset = 0;
 			}
@@ -924,7 +926,7 @@ process_next:
 		icnt++;
 		cluster_offset++;
 
-		if (icnt == mp->m_ialloc_inos &&
+		if (icnt == igeo->ialloc_inos &&
 				irec_offset == XFS_INODES_PER_CHUNK)  {
 			/*
 			 * done! - finished up irec and block simultaneously
@@ -962,7 +964,7 @@ process_next:
 			ASSERT(ino_rec->ino_startnum == agino + 1);
 			irec_offset = 0;
 		}
-		if (cluster_offset == inodes_per_cluster) {
+		if (cluster_offset == M_IGEO(mp)->inodes_per_cluster) {
 			bp_index++;
 			cluster_offset = 0;
 		}
@@ -993,6 +995,7 @@ process_aginodes(
 {
 	int 			num_inos, bogus;
 	ino_tree_node_t 	*ino_rec, *first_ino_rec, *prev_ino_rec;
+	struct xfs_ino_geometry *igeo = M_IGEO(mp);
 #ifdef XR_PF_TRACE
 	int			count;
 #endif
@@ -1008,7 +1011,7 @@ process_aginodes(
 		 * the next block before we call the processing routines.
 		 */
 		num_inos = XFS_INODES_PER_CHUNK;
-		while (num_inos < mp->m_ialloc_inos && ino_rec != NULL)  {
+		while (num_inos < igeo->ialloc_inos && ino_rec != NULL)  {
 			/*
 			 * inodes chunks will always be aligned and sized
 			 * correctly
@@ -1017,7 +1020,7 @@ process_aginodes(
 				num_inos += XFS_INODES_PER_CHUNK;
 		}
 
-		ASSERT(num_inos == mp->m_ialloc_inos);
+		ASSERT(num_inos == igeo->ialloc_inos);
 
 		if (pf_args) {
 			sem_post(&pf_args->ra_count);
@@ -1049,7 +1052,7 @@ process_aginodes(
 			 */
 			num_inos = 0;
 			ino_rec = first_ino_rec;
-			while (num_inos < mp->m_ialloc_inos &&
+			while (num_inos < igeo->ialloc_inos &&
 					ino_rec != NULL)  {
 				prev_ino_rec = ino_rec;
 
@@ -1175,6 +1178,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 	int			bogus;
 	int			cnt;
 	int			got_some;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 #ifdef XR_INODE_TRACE
 	fprintf(stderr, "in process_uncertain_aginodes, agno = %d\n", agno);
@@ -1233,7 +1237,7 @@ process_uncertain_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
 			 * processing may add more records to the
 			 * uncertain inode lists.
 			 */
-			if (process_inode_chunk(mp, agno, mp->m_ialloc_inos,
+			if (process_inode_chunk(mp, agno, igeo->ialloc_inos,
 						nrec, 1, 0, 0, &bogus))  {
 				/* XXX - i/o error, we've got a problem */
 				abort();
