@@ -9,7 +9,7 @@
 #include <sys/resource.h>
 #include "xfs_multidisk.h"
 #include "avl.h"
-#include "avl64.h"
+#include "libfrog/avl64.h"
 #include "globals.h"
 #include "versions.h"
 #include "agheader.h"
@@ -22,6 +22,8 @@
 #include "dinode.h"
 #include "slab.h"
 #include "rmap.h"
+#include "libfrog/fsgeom.h"
+#include "libfrog/platform.h"
 
 /*
  * option tables for getsubopt calls
@@ -399,7 +401,7 @@ calc_mkfs(xfs_mount_t *mp)
 	xfs_agblock_t	fino_bno;
 	int		do_inoalign;
 
-	do_inoalign = mp->m_sinoalign;
+	do_inoalign = M_IGEO(mp)->ialloc_align;
 
 	/*
 	 * Pre-calculate the geometry of ag 0. We know what it looks like
@@ -458,9 +460,9 @@ calc_mkfs(xfs_mount_t *mp)
 		first_prealloc_ino = XFS_AGB_TO_AGINO(mp, fino_bno);
 	}
 
-	ASSERT(mp->m_ialloc_blks > 0);
+	ASSERT(M_IGEO(mp)->ialloc_blks > 0);
 
-	if (mp->m_ialloc_blks > 1)
+	if (M_IGEO(mp)->ialloc_blks > 1)
 		last_prealloc_ino = first_prealloc_ino + XFS_INODES_PER_CHUNK;
 	else
 		last_prealloc_ino = XFS_AGB_TO_AGINO(mp, fino_bno + 1);
@@ -634,13 +636,14 @@ static void
 check_fs_vs_host_sectsize(
 	struct xfs_sb	*sb)
 {
-	int	fd;
+	int	fd, ret;
 	long	old_flags;
-	struct xfs_fsop_geom_v1 geom = { 0 };
+	struct xfs_fsop_geom	geom = { 0 };
 
 	fd = libxfs_device_to_fd(x.ddev);
 
-	if (ioctl(fd, XFS_IOC_FSGEOMETRY_V1, &geom) < 0) {
+	ret = -xfrog_geometry(fd, &geom);
+	if (ret) {
 		do_log(_("Cannot get host filesystem geometry.\n"
 	"Repair may fail if there is a sector size mismatch between\n"
 	"the image and the host filesystem.\n"));
@@ -670,6 +673,7 @@ main(int argc, char **argv)
 	char		*msgbuf;
 	struct xfs_sb	psb;
 	int		rval;
+	struct xfs_ino_geometry	*igeo;
 
 	progname = basename(argv[0]);
 	setlocale(LC_ALL, "");
@@ -747,6 +751,7 @@ main(int argc, char **argv)
 		exit(1);
 	}
 	mp->m_log = &log;
+	igeo = M_IGEO(mp);
 
 	/* Spit out function & line on these corruption macros */
 	if (verbose > 2)
@@ -759,8 +764,6 @@ main(int argc, char **argv)
 
 	chunks_pblock = mp->m_sb.sb_inopblock / XFS_INODES_PER_CHUNK;
 	max_symlink_blocks = libxfs_symlink_blocks(mp, XFS_SYMLINK_MAXLEN);
-	inodes_per_cluster = max(mp->m_sb.sb_inopblock,
-			mp->m_inode_cluster_size >> mp->m_sb.sb_inodelog);
 
 	/*
 	 * Automatic striding for high agcount filesystems.
@@ -850,7 +853,7 @@ main(int argc, char **argv)
 					(mp->m_sb.sb_dblocks >> (10 + 1)) +
 					50000;	/* rough estimate of 50MB overhead */
 		max_mem = max_mem_specified ? max_mem_specified * 1024 :
-						libxfs_physmem() * 3 / 4;
+					      platform_physmem() * 3 / 4;
 
 		if (getrlimit(RLIMIT_AS, &rlim) != -1 &&
 					rlim.rlim_cur != RLIM_INFINITY) {
@@ -896,7 +899,7 @@ main(int argc, char **argv)
 		if (max_mem >= (1 << 30))
 			max_mem = 1 << 30;
 		libxfs_bhash_size = max_mem / (HASH_CACHE_RATIO *
-				(mp->m_inode_cluster_size >> 10));
+				(igeo->inode_cluster_size >> 10));
 		if (libxfs_bhash_size < 512)
 			libxfs_bhash_size = 512;
 
@@ -1044,7 +1047,7 @@ _("Warning:  project quota information would be cleared.\n"
 	/*
 	 * Clear the quota flags if they're on.
 	 */
-	sbp = libxfs_getsb(mp, 0);
+	sbp = libxfs_getsb(mp);
 	if (!sbp)
 		do_error(_("couldn't get superblock\n"));
 
